@@ -45,12 +45,14 @@ int set_essid(struct wrap_ndis_device *wnd, const char *ssid, int ssid_len)
 		return -EINVAL;
 
 	memset(&req, 0, sizeof(req));
-	if (ssid_len == 0)
-		req.length = 1;
-	else {
-		req.length = ssid_len;
-		memcpy(&req.essid, ssid, req.length);
-		DBGTRACE2("ssid = '%s'", req.essid);
+	req.length = ssid_len;
+	if (ssid_len)
+		memcpy(&req.essid, ssid, ssid_len);
+	DBG_BLOCK(2) {
+		char buf[NDIS_ESSID_MAX_SIZE+1];
+		memcpy(buf, ssid, ssid_len);
+		buf[ssid_len] = 0;
+		DBGTRACE2("ssid = '%s'", buf);
 	}
 
 	res = miniport_set_info(wnd, OID_802_11_SSID, &req, sizeof(req));
@@ -124,25 +126,27 @@ static int iw_set_essid(struct net_device *dev, struct iw_request_info *info,
 			union iwreq_data *wrqu, char *extra)
 {
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
-	char ssid[IW_ESSID_MAX_SIZE];
+	char ssid[NDIS_ESSID_MAX_SIZE];
 	int length;
 
 	TRACEENTER2("");
-	if (wrqu->essid.length > IW_ESSID_MAX_SIZE)
-		TRACEEXIT2(return -EINVAL);
-
+	memset(ssid, 0, sizeof(ssid));
 	/* there is no way to turn off essid other than to set to
 	 * random bytes; instead, we use off to mean any */
 	if (wrqu->essid.flags) {
 		/* wireless-tools prior to version 20 add extra 1, and
 		 * later than 20 don't! Deal with that mess */
-		for (length = 0; length < wrqu->essid.length; length++)
-			if (!extra[length])
-				break;
+		length = wrqu->essid.length - 1;
+		if (length > 0)
+			length--;
+		while (length < wrqu->essid.length && extra[length])
+			length++;
+		DBGTRACE2("%d", length);
+		if (length <= 0 || length > NDIS_ESSID_MAX_SIZE)
+			TRACEEXIT2(return -EINVAL);
 	} else
 		length = 0;
 
-	memset(ssid, 0, sizeof(ssid));
 	if (wnd->iw_auth_set) {
 		int ret = set_assoc_params(wnd);
 		wnd->iw_auth_set = 0;
@@ -383,8 +387,7 @@ static int iw_set_freq(struct net_device *dev, struct iw_request_info *info,
 	return 0;
 }
 
-static int iw_get_tx_power(struct net_device *dev,
-			   struct iw_request_info *info,
+static int iw_get_tx_power(struct net_device *dev, struct iw_request_info *info,
 			   union iwreq_data *wrqu, char *extra)
 {
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
@@ -403,8 +406,7 @@ static int iw_get_tx_power(struct net_device *dev,
 	return 0;
 }
 
-static int iw_set_tx_power(struct net_device *dev,
-			   struct iw_request_info *info,
+static int iw_set_tx_power(struct net_device *dev, struct iw_request_info *info,
 			   union iwreq_data *wrqu, char *extra)
 {
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
@@ -838,11 +840,8 @@ static int remove_key(struct wrap_ndis_device *wnd, int index,
 	wnd->encr_info.keys[index].length = 0;
 	memset(&wnd->encr_info.keys[index].key, 0,
 	       sizeof(wnd->encr_info.keys[index].length));
-	/* TI driver crashes kernel if OID_802_11_REMOVE_KEY is
-	 * called; other drivers seem to not require it, so for now,
-	 * don't remove the key from driver */
 	if (wnd->encr_mode == Ndis802_11Encryption2Enabled ||
-	    wnd->encr_mode == Ndis802_11Encryption2Enabled) {
+	    wnd->encr_mode == Ndis802_11Encryption3Enabled) {
 		struct ndis_remove_key remove_key;
 		remove_key.struct_size = sizeof(remove_key);
 		remove_key.index = index;
@@ -1052,9 +1051,14 @@ static char *ndis_translate_scan(struct net_device *dev, char *event,
 	/* add qual */
 	memset(&iwe, 0, sizeof(iwe));
 	iwe.cmd = IWEVQUAL;
+	i = 100 * (bssid->rssi - WL_NOISE) / (WL_SIGMAX - WL_NOISE);
+	if (i < 0)
+		i = 0;
+	else if (i > 100)
+		i = 100;
 	iwe.u.qual.level = bssid->rssi;
-	iwe.u.qual.noise = 0;
-	iwe.u.qual.qual = 0;
+	iwe.u.qual.noise = WL_NOISE;
+	iwe.u.qual.qual  = i;
 	iwe.len = IW_EV_QUAL_LEN;
 	event = iwe_stream_add_event(event, end_buf, &iwe, IW_EV_QUAL_LEN);
 
@@ -1237,8 +1241,8 @@ static int iw_get_scan(struct net_device *dev, struct iw_request_info *info,
 	DBGTRACE2("%d", bssid_list->num_items);
 	cur_item = &bssid_list->bssid[0];
 	for (i = 0; i < bssid_list->num_items; i++) {
-		event = ndis_translate_scan(dev, event, extra + IW_SCAN_MAX_DATA,
-					    cur_item);
+		event = ndis_translate_scan(dev, event,
+					    extra + IW_SCAN_MAX_DATA, cur_item);
 		cur_item = (struct ndis_wlan_bssid *)((char *)cur_item +
 						      cur_item->length);
 	}
@@ -1661,14 +1665,14 @@ static int iw_set_encodeext(struct net_device *dev,
 
 	memset(&ndis_key, 0, sizeof(ndis_key));
 
-	ndis_key.struct_size = sizeof(ndis_key);
+	ndis_key.struct_size =
+		sizeof(ndis_key) - sizeof(ndis_key.key) + ext.key_len;
 	ndis_key.length = ext.key_len;
 	ndis_key.index = keyidx;
 
 	if (ext.ext_flags & IW_ENCODE_EXT_RX_SEQ_VALID) {
-		for (i = 0, ndis_key.rsc = 0 ; i < 6 ; i++)
+		for (i = ndis_key.rsc = 0 ; i < 6 ; i++)
 			ndis_key.rsc |= (ext.rx_seq[i] << (i * 8));
-
 		ndis_key.index |= 1 << 29;
 	}
 
@@ -1703,7 +1707,7 @@ static int iw_set_encodeext(struct net_device *dev,
 		memcpy(ndis_key.key, key, ext.key_len);
 
 	res = miniport_set_info(wnd, OID_802_11_ADD_KEY,
-				&ndis_key, sizeof(ndis_key));
+				&ndis_key, ndis_key.struct_size);
 	if (res) {
 		DBGTRACE2("adding key failed (%08X), %u",
 			  res, ndis_key.struct_size);
@@ -1729,8 +1733,7 @@ static int iw_get_encodeext(struct net_device *dev,
 	return 0;
 }
 
-static int iw_set_pmksa(struct net_device *dev,
-			struct iw_request_info *info,
+static int iw_set_pmksa(struct net_device *dev, struct iw_request_info *info,
 			union iwreq_data *wrqu, char *extra)
 {
 	struct iw_pmksa *pmksa = (struct iw_pmksa *) extra;
@@ -2040,8 +2043,7 @@ static int wpa_set_key(struct net_device *dev, struct iw_request_info *info,
 	if (wpa_key.key && copy_from_user(&key, wpa_key.key, wpa_key.key_len))
 		TRACEEXIT2(return -EFAULT);
 
-	TRACEENTER2("alg = %d, key_index = %d",
-		    wpa_key.alg, wpa_key.key_index);
+	TRACEENTER2("alg = %d, key_index = %d", wpa_key.alg, wpa_key.key_index);
 
 	if (wpa_key.alg == WPA_ALG_WEP) {
 		if (!test_bit(Ndis802_11Encryption1Enabled, &wnd->capa.encr))
@@ -2058,15 +2060,14 @@ static int wpa_set_key(struct net_device *dev, struct iw_request_info *info,
 		TRACEEXIT2(return -1);
 	}
 	if (wpa_key.seq_len > IW_ENCODING_TOKEN_MAX) {
-		DBGTRACE2("incorrect seq? length = (%u)",
-			  (u32)wpa_key.seq_len);
+		DBGTRACE2("incorrect seq? length = (%u)", (u32)wpa_key.seq_len);
 		TRACEEXIT2(return -1);
 	}
-	DBGTRACE2("setting key %d, %u", wpa_key.key_index,
-		  (u32)wpa_key.key_len);
+	DBGTRACE2("setting key %d, %u", wpa_key.key_index, (u32)wpa_key.key_len);
 	memset(&ndis_key, 0, sizeof(ndis_key));
 
-	ndis_key.struct_size = sizeof(ndis_key);
+	ndis_key.struct_size =
+		sizeof(ndis_key) - sizeof(ndis_key.key) + wpa_key.key_len;
 	ndis_key.length = wpa_key.key_len;
 	ndis_key.index = wpa_key.key_index;
 	if (wpa_key.seq && wpa_key.seq_len > 0) {
@@ -2111,7 +2112,7 @@ static int wpa_set_key(struct net_device *dev, struct iw_request_info *info,
 		DBGTRACE2("key %d removed", wpa_key.key_index);
 	} else {
 		res = miniport_set_info(wnd, OID_802_11_ADD_KEY,
-					&ndis_key, sizeof(ndis_key));
+					&ndis_key, ndis_key.struct_size);
 		if (res) {
 			DBGTRACE2("adding key failed (%08X), %u",
 				  res, ndis_key.struct_size);
