@@ -55,8 +55,6 @@ static struct usb_device_id wrap_usb_device;
 struct usb_driver wrap_usb_driver;
 #endif
 
-int load_pe_images(struct pe_image[], int n);
-
 int wrap_device_type(int data1)
 {
 	int i;
@@ -83,7 +81,7 @@ struct wrap_driver *load_wrap_driver(struct wrap_device *wd)
 	wrap_driver = NULL;
 	nt_list_for_each(cur, &wrap_drivers) {
 		wrap_driver = container_of(cur, struct wrap_driver, list);
-		if (!strcmp(wrap_driver->name, wd->driver_name)) {
+		if (!stricmp(wrap_driver->name, wd->driver_name)) {
 			DBGTRACE1("driver %s already loaded",
 				  wrap_driver->name);
 			break;
@@ -126,7 +124,7 @@ struct wrap_driver *load_wrap_driver(struct wrap_device *wd)
 		nt_list_for_each(cur, &wrap_drivers) {
 			wrap_driver = container_of(cur, struct wrap_driver,
 						   list);
-			if (!strcmp(wrap_driver->name, wd->driver_name)) {
+			if (!stricmp(wrap_driver->name, wd->driver_name)) {
 				wd->driver = wrap_driver;
 				break;
 			} else
@@ -149,7 +147,8 @@ static int load_sys_files(struct wrap_driver *driver,
 
 	DBGTRACE1("num_pe_images = %d", load_driver->nr_sys_files);
 	DBGTRACE1("loading driver: %s", load_driver->name);
-	strncpy(driver->name, load_driver->name, MAX_DRIVER_NAME_LEN);
+	strncpy(driver->name, load_driver->name, sizeof(driver->name));
+	driver->name[sizeof(driver->name)-1] = 0;
 	DBGTRACE1("driver: %s", driver->name);
 	err = 0;
 	driver->num_pe_images = 0;
@@ -196,6 +195,7 @@ static int load_sys_files(struct wrap_driver *driver,
 #endif
 		if (!pe_image->image) {
 			ERROR("couldn't allocate memory");
+			err = -ENOMEM;
 			break;
 		}
 		DBGTRACE1("image is at %p", pe_image->image);
@@ -205,13 +205,14 @@ static int load_sys_files(struct wrap_driver *driver,
 				   load_driver->sys_files[i].size)) {
 			ERROR("couldn't load file %s",
 			      load_driver->sys_files[i].name);
+			err = -EFAULT;
 			break;
 		}
 		pe_image->size = load_driver->sys_files[i].size;
 		driver->num_pe_images++;
 	}
 
-	if (load_pe_images(driver->pe_images, driver->num_pe_images)) {
+	if (!err && link_pe_images(driver->pe_images, driver->num_pe_images)) {
 		ERROR("couldn't prepare driver '%s'", load_driver->name);
 		err = -EINVAL;
 	}
@@ -221,7 +222,7 @@ static int load_sys_files(struct wrap_driver *driver,
 			if (driver->pe_images[i].image)
 				vfree(driver->pe_images[i].image);
 		driver->num_pe_images = 0;
-		TRACEEXIT1(return -EINVAL);
+		TRACEEXIT1(return err);
 	} else
 		TRACEEXIT1(return 0);
 }
@@ -231,6 +232,7 @@ struct wrap_bin_file *get_bin_file(char *bin_file_name)
 	int i = 0;
 	struct wrap_driver *driver, *cur;
 
+	TRACEENTER1("%s", bin_file_name);
 	if (down_interruptible(&loader_mutex)) {
 		WARNING("couldn't obtain loader_mutex");
 		TRACEEXIT1(return NULL);
@@ -242,11 +244,11 @@ struct wrap_bin_file *get_bin_file(char *bin_file_name)
 				driver = cur;
 				break;
 			}
-		if (i < cur->num_bin_files)
+		if (driver)
 			break;
 	}
 	up(&loader_mutex);
-	if (driver == NULL) {
+	if (!driver) {
 		DBGTRACE1("coudln't find bin file '%s'", bin_file_name);
 		return NULL;
 	}
@@ -259,12 +261,12 @@ struct wrap_bin_file *get_bin_file(char *bin_file_name)
 				"0",
 #endif
 				UTILS_VERSION, driver->name,
-				bin_file_name, NULL};
+				driver->bin_files[i].name, NULL};
 		char *env[] = {NULL};
 		int ret;
 
 		DBGTRACE1("loading bin file %s/%s", driver->name,
-			  bin_file_name);
+			  driver->bin_files[i].name);
 		if (down_interruptible(&loader_mutex)) {
 			WARNING("couldn't obtain loader_mutex");
 			TRACEEXIT1(return NULL);
@@ -283,8 +285,11 @@ struct wrap_bin_file *get_bin_file(char *bin_file_name)
 		}
 		wait_for_completion(&loader_complete);
 		up(&loader_mutex);
-		if (!driver->bin_files[i].data)
+		if (!driver->bin_files[i].data) {
+			WARNING("couldn't load binary file %s",
+				driver->bin_files[i].name);
 			TRACEEXIT1(return NULL);
+		}
 	}
 	TRACEEXIT2(return &(driver->bin_files[i]));
 }
@@ -304,7 +309,7 @@ static int add_bin_file(struct load_driver_file *driver_file)
 				driver = cur;
 				break;
 			}
-		if (i < cur->num_bin_files)
+		if (driver)
 			break;
 	}
 	if (!driver) {
@@ -314,14 +319,15 @@ static int add_bin_file(struct load_driver_file *driver_file)
 	bin_file = &driver->bin_files[i];
 	strncpy(bin_file->name, driver_file->name, sizeof(bin_file->name));
 	bin_file->name[sizeof(bin_file->name)-1] = 0;
-	bin_file->size = driver_file->size;
-	bin_file->data = vmalloc(bin_file->size);
+	bin_file->data = vmalloc(driver_file->size);
 	if (!bin_file->data) {
 		ERROR("couldn't allocate memory");
 		return -ENOMEM;
 	}
+	bin_file->size = driver_file->size;
 	if (copy_from_user(bin_file->data, driver_file->data, bin_file->size)) {
 		ERROR("couldn't copy data");
+		free_bin_file(bin_file);
 		return -EFAULT;
 	}
 	return 0;
@@ -376,7 +382,7 @@ static int load_settings(struct wrap_driver *wrap_driver,
 	wd = NULL;
 	nt_list_for_each(cur, &wrap_devices) {
 		wd = container_of(cur, struct wrap_device, list);
-		if (strcmp(wd->conf_file_name, load_driver->conf_file_name))
+		if (stricmp(wd->conf_file_name, load_driver->conf_file_name))
 			wd = NULL;
 		else
 			break;
@@ -530,7 +536,7 @@ static int add_wrap_driver(struct wrap_driver *driver)
 
 	TRACEENTER1("name: %s", driver->name);
 	nt_list_for_each_entry(tmp, &wrap_drivers, list) {
-		if (strcmp(tmp->name, driver->name) == 0) {
+		if (stricmp(tmp->name, driver->name) == 0) {
 			ERROR("cannot add duplicate driver");
 			TRACEEXIT1(return -EBUSY);
 		}
@@ -583,8 +589,8 @@ static int load_user_space_driver(struct load_driver *load_driver)
 		ObDereferenceObject(drv_obj);
 		TRACEEXIT1(return -EINVAL);
 	}
-	strncpy(wrap_driver->name, load_driver->name,
-		sizeof(wrap_driver->name));
+	strncpy(wrap_driver->name, load_driver->name, sizeof(wrap_driver->name));
+	wrap_driver->name[sizeof(wrap_driver->name)-1] = 0;
 	if (load_sys_files(wrap_driver, load_driver) ||
 	    load_bin_files_info(wrap_driver, load_driver) ||
 	    load_settings(wrap_driver, load_driver) ||
@@ -709,8 +715,8 @@ struct wrap_device *load_wrap_device(struct load_device *load_device)
 			);
 		if (ret) {
 			up(&loader_mutex);
-			DBGTRACE1("loadndiswrapper failed (%d); check system log "
-				  "for messages from 'loadndisdriver'", ret);
+			DBGTRACE1("loadndiswrapper failed (%d); check system "
+				  "log for messages from 'loadndisdriver'", ret);
 			TRACEEXIT1(return NULL);
 		}
 		wait_for_completion(&loader_complete);
