@@ -22,9 +22,9 @@
 #define MAX_ALLOCATED_NDIS_PACKETS 20
 #define MAX_ALLOCATED_NDIS_BUFFERS 20
 
-static struct workqueue_struct *ndis_wq;
+static workqueue_struct_t *ndis_wq;
 static void ndis_worker(void *dummy);
-static struct work_struct ndis_work;
+static work_struct_t ndis_work;
 static struct nt_list ndis_worker_list;
 static NT_SPIN_LOCK ndis_work_list_lock;
 
@@ -33,14 +33,10 @@ extern struct semaphore loader_mutex;
 /* ndis_init is called once when module is loaded */
 int ndis_init(void)
 {
-#ifdef USE_OWN_WORKQUEUE
 	ndis_wq = create_singlethread_workqueue("ndis_wq");
-#else
-	ndis_wq = NULL;
-#endif
 	InitializeListHead(&ndis_worker_list);
 	nt_spin_lock_init(&ndis_work_list_lock);
-	INIT_WORK(&ndis_work, ndis_worker, NULL);
+	initialize_work(&ndis_work, ndis_worker, NULL);
 
 	return 0;
 }
@@ -48,9 +44,7 @@ int ndis_init(void)
 /* ndis_exit is called once when module is removed */
 void ndis_exit(void)
 {
-#ifdef USE_OWN_WORKQUEUE
 	destroy_workqueue(ndis_wq);
-#endif
 	TRACEEXIT1(return);
 }
 
@@ -237,7 +231,7 @@ noregparm void WIN_FUNC(NdisWriteErrorLogEntry,12)
 	      error, count, __builtin_return_address(0));
 	for (i = 0; i < count; i++) {
 		code = va_arg(args, ULONG);
-		ERROR("code: %u", code);
+		ERROR("code: 0x%x", code);
 	}
 	va_end(args);
 	TRACEEXIT2(return);
@@ -369,11 +363,26 @@ wstdcall ULONG WIN_FUNC(NDIS_BUFFER_TO_SPAN_PAGES,1)
 	ULONG n, length;
 
 	if (buffer == NULL)
-		return 0;
+		TRACEEXIT2(return 0);
 	if (MmGetMdlByteCount(buffer) == 0)
-		return 1;
+		TRACEEXIT2(return 1);
 	length = MmGetMdlByteCount(buffer);
+
+#ifdef VT6655
+	/* VIA VT6655 works with this bogus computation, but not with
+	 * correct computation with SPAN_PAGES */
+	if (1) {
+		ULONG_PTR start, end;
+		unsigned long ptr;
+
+		ptr = (unsigned long)MmGetMdlVirtualAddress(buffer);
+		start = ptr & (PAGE_SIZE - 1);
+		end = (ptr + length + PAGE_SIZE - 1) & PAGE_MASK;
+		n = (end - start) / PAGE_SIZE;
+	}
+#else
 	n = SPAN_PAGES(MmGetMdlVirtualAddress(buffer), length);
+#endif
 	DBGTRACE4("%p, %p, %d, %d", buffer->startva, buffer->mappedsystemva,
 		  length, n);
 	TRACEEXIT3(return n);
@@ -723,9 +732,8 @@ wstdcall void WIN_FUNC(NdisMQueryAdapterResources,4)
 	resource_length = sizeof(struct cm_partial_resource_list) +
 		sizeof(struct cm_partial_resource_descriptor) *
 		(list->count - 1);
-	DBGTRACE2("%p, %p,%d (%d), %p %d %d", wnd,
-		  resource_list, *size, resource_length,
-		  &list->partial_descriptors[list->count-1],
+	DBGTRACE2("%p, %p,%d (%d), %p %d %d", wnd, resource_list, *size,
+		  resource_length, &list->partial_descriptors[list->count-1],
 		  list->partial_descriptors[list->count-1].u.interrupt.level,
 		  list->partial_descriptors[list->count-1].u.interrupt.vector);
 	if (*size < sizeof(*list)) {
@@ -849,29 +857,33 @@ wstdcall NDIS_STATUS WIN_FUNC(NdisMAllocateMapRegisters,5)
 	 NDIS_DMA_SIZE dmasize, ULONG basemap, ULONG max_buf_size)
 {
 	struct wrap_ndis_device *wnd = nmb->wnd;
-	TRACEENTER2("%d %d %d %d", dmachan, dmasize, basemap, max_buf_size);
 
+	TRACEENTER2("%p, %d %d %d %d",
+		    wnd, dmachan, dmasize, basemap, max_buf_size);
+	if (wnd->dma_map_count > 0) {
+		WARNING("%s: map registers already allocated: %u",
+			wnd->net_dev->name, wnd->dma_map_count);
+		TRACEEXIT2(return NDIS_STATUS_RESOURCES);
+	}
+	/* since memory for buffer is allocated with kmalloc, buffer
+	 * is physically contiguous, so entire map will fit in one
+	 * register */
 	if (basemap > 64) {
-		WARNING("invalid request: %d", basemap);
-		TRACEEXIT1(return NDIS_STATUS_RESOURCES);
+		WARNING("Windows driver %s requesting too many (%u) "
+			"map registers", wnd->wd->driver->name, basemap);
+		/* As per NDIS, NDIS_STATUS_RESOURCES should be
+		 * retrned, but with that Atheros PCI driver fails -
+		 * for now tolerate it */
+//		TRACEEXIT2(return NDIS_STATUS_RESOURCES);
 	}
 
-	/* TODO: should we allocate max_buf_size % PAGE_SIZE + 1
-	 * entries for each map? */
-	if (max_buf_size > PAGE_SIZE)
-		WARNING("buffer size too big: %u", max_buf_size);
-
-	if (wnd->tx_dma_count > 0) {
-		DBGTRACE2("%s: map registers already allocated: %u",
-			  wnd->net_dev->name, wnd->tx_dma_count);
+	wnd->dma_map_addr = kmalloc(basemap * sizeof(*(wnd->dma_map_addr)),
+				    GFP_KERNEL);
+	if (!wnd->dma_map_addr)
 		TRACEEXIT2(return NDIS_STATUS_RESOURCES);
-	}
-	wnd->tx_dma_count = basemap;
-	wnd->tx_dma_addr = kmalloc(basemap * sizeof(dma_addr_t),
-				   GFP_KERNEL);
-	if (!wnd->tx_dma_addr)
-		TRACEEXIT2(return NDIS_STATUS_RESOURCES);
-	memset(wnd->tx_dma_addr, 0, basemap * sizeof(dma_addr_t));
+	memset(wnd->dma_map_addr, 0, basemap * sizeof(*(wnd->dma_map_addr)));
+	wnd->dma_map_count = basemap;
+	DBGTRACE2("%u", wnd->dma_map_count);
 	TRACEEXIT2(return NDIS_STATUS_SUCCESS);
 }
 
@@ -879,20 +891,21 @@ wstdcall void WIN_FUNC(NdisMFreeMapRegisters,1)
 	(struct ndis_miniport_block *nmb)
 {
 	struct wrap_ndis_device *wnd = nmb->wnd;
-	TRACEENTER2("wnd: %p", wnd);
+	int i;
 
-	if (wnd->tx_dma_addr) {
-		u8 i;
-		for (i = 0; i < wnd->tx_dma_count; i++) {
-			if (wnd->tx_dma_addr[i])
+	TRACEENTER2("wnd: %p", wnd);
+	if (wnd->dma_map_addr) {
+		for (i = 0; i < wnd->dma_map_count; i++) {
+			if (wnd->dma_map_addr[i])
 				WARNING("%s: dma addr %p not freed by "
 					"Windows driver", wnd->net_dev->name,
-					(void *)wnd->tx_dma_addr[i]);
+					(void *)wnd->dma_map_addr[i]);
 		}
-		kfree(wnd->tx_dma_addr);
-		wnd->tx_dma_addr = NULL;
-	}
-	wnd->tx_dma_count = 0;
+		kfree(wnd->dma_map_addr);
+		wnd->dma_map_addr = NULL;
+	} else
+		WARNING("map registers already freed?");
+	wnd->dma_map_count = 0;
 	TRACEEXIT2(return);
 }
 
@@ -902,23 +915,21 @@ wstdcall void WIN_FUNC(NdisMStartBufferPhysicalMapping,6)
 	 struct ndis_phy_addr_unit *phy_addr_array, UINT *array_size)
 {
 	struct wrap_ndis_device *wnd = nmb->wnd;
-	TRACEENTER3("index: %u", index);
 
-	if (wnd->use_sg_dma)
-		WARNING("buffer %p must have been mapped already", buf);
-	if (!write_to_dev) {
-		ERROR("invalid dma direction (%d)", write_to_dev);
+	TRACEENTER3("%p, %p, %u, %u", wnd, buf, index, wnd->dma_map_count);
+	if (wnd->use_sg_dma || !write_to_dev ||
+	    index >= wnd->dma_map_count) {
+		WARNING("invalid request: %d, %d, %d, %d", wnd->use_sg_dma,
+			write_to_dev, index, wnd->dma_map_count);
+		phy_addr_array[0].phy_addr = 0;
+		phy_addr_array[0].length = 0;
 		*array_size = 0;
 		return;
 	}
-	if (index >= wnd->tx_dma_count) {
-		ERROR("invalid map register (%u >= %u)",
-		      index, wnd->tx_dma_count);
-		*array_size = 0;
-		return;
-	}
-	if (wnd->tx_dma_addr[index] != 0) {
-		*array_size = 1;
+	if (wnd->dma_map_addr[index]) {
+		DBGTRACE2("buffer %p at %d is already mapped: %lx", buf, index,
+			  (unsigned long)wnd->dma_map_addr[index]);
+//		*array_size = 1;
 		return;
 	}
 	DBGTRACE3("%p, %p, %u", buf, MmGetSystemAddressForMdl(buf),
@@ -927,39 +938,38 @@ wstdcall void WIN_FUNC(NdisMStartBufferPhysicalMapping,6)
 		dump_bytes(__FUNCTION__, MmGetSystemAddressForMdl(buf),
 			   MmGetMdlByteCount(buf));
 	}
-	phy_addr_array[0].phy_addr =
+	wnd->dma_map_addr[index] = 
 		PCI_DMA_MAP_SINGLE(wnd->wd->pci.pdev,
 				   MmGetSystemAddressForMdl(buf),
 				   MmGetMdlByteCount(buf), PCI_DMA_TODEVICE);
+	phy_addr_array[0].phy_addr = wnd->dma_map_addr[index];
 	phy_addr_array[0].length = MmGetMdlByteCount(buf);
-	*array_size = 1;
 	DBGTRACE4("%Lx, %d, %d", phy_addr_array[0].phy_addr,
 		  phy_addr_array[0].length, index);
-	wnd->tx_dma_addr[index] = phy_addr_array[0].phy_addr;
+	*array_size = 1;
 }
 
 wstdcall void WIN_FUNC(NdisMCompleteBufferPhysicalMapping,3)
 	(struct ndis_miniport_block *nmb, ndis_buffer *buf, ULONG index)
 {
 	struct wrap_ndis_device *wnd = nmb->wnd;
-	TRACEENTER3("%p, %p %u (%u)", wnd, buf, index, wnd->tx_dma_count);
+
+	TRACEENTER3("%p, %p %u (%u)", wnd, buf, index, wnd->dma_map_count);
 
 	if (wnd->use_sg_dma)
 		WARNING("buffer %p may have been unmapped already", buf);
-	if (index >= wnd->tx_dma_count) {
+	if (index >= wnd->dma_map_count) {
 		ERROR("invalid map register (%u >= %u)",
-		      index, wnd->tx_dma_count);
+		      index, wnd->dma_map_count);
 		return;
 	}
-	DBGTRACE4("%lx, %d, %d", (unsigned long)wnd->tx_dma_addr[index],
-		  MmGetMdlByteCount(buf), index);
-	if (wnd->tx_dma_addr[index] == 0) {
-//		ERROR("map register not used (%lu)", index);
-		return;
-	}
-	PCI_DMA_UNMAP_SINGLE(wnd->wd->pci.pdev, wnd->tx_dma_addr[index],
-			     MmGetMdlByteCount(buf), PCI_DMA_TODEVICE);
-	wnd->tx_dma_addr[index] = 0;
+	DBGTRACE4("%lx", (unsigned long)wnd->dma_map_addr[index]);
+	if (wnd->dma_map_addr[index]) {
+		PCI_DMA_UNMAP_SINGLE(wnd->wd->pci.pdev, wnd->dma_map_addr[index],
+				     MmGetMdlByteCount(buf), PCI_DMA_TODEVICE);
+		wnd->dma_map_addr[index] = 0;
+	} else
+		WARNING("map registers at %u not used", index);
 }
 
 wstdcall void WIN_FUNC(NdisMAllocateSharedMemory,5)
@@ -1087,13 +1097,14 @@ wstdcall void WIN_FUNC(NdisAllocateBuffer,5)
 				pool->num_allocated_descr, pool->max_descr);
 	}
 	descr = atomic_remove_list_head(pool->free_descr, oldhead->next);
+	/* TODO: make sure this mdl can map given buffer */
 	if (descr) {
-		typeof(descr->flags) flags;
-		flags = descr->flags;
+		typeof(descr->flags) flags = descr->flags;
 		MmInitializeMdl(descr, virt, length);
-		if (flags & MDL_CACHE_ALLOCATED)
-			descr->flags = MDL_CACHE_ALLOCATED;
+//		MmBuildMdlForNonPagedPool(descr);
 		descr->flags |= MDL_SOURCE_IS_NONPAGED_POOL;
+		if (flags & MDL_CACHE_ALLOCATED)
+			descr->flags |= MDL_CACHE_ALLOCATED;
 	} else {
 		descr = allocate_init_mdl(virt, length);
 		if (!descr) {
@@ -1106,6 +1117,8 @@ wstdcall void WIN_FUNC(NdisAllocateBuffer,5)
 		atomic_inc_var(pool->num_allocated_descr);
 		MmBuildMdlForNonPagedPool(descr);
 	}
+//	descr->flags |= MDL_ALLOCATED_FIXED_SIZE |
+//		MDL_MAPPED_TO_SYSTEM_VA | MDL_PAGES_LOCKED;
 	descr->pool = pool;
 	*buffer = descr;
 	*status = NDIS_STATUS_SUCCESS;
@@ -1745,9 +1758,9 @@ static void ndis_irq_handler(unsigned long data)
 	if_serialize_unlock(wnd);
 }
 
-irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs)
+irqreturn_t ndis_isr(int irq, void *data ISR_PT_REGS_PARAM_DECL)
 {
-	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
+	struct wrap_ndis_device *wnd = data;
 	struct miniport_char *miniport;
 	BOOLEAN recognized, queue_handler;
 
