@@ -798,7 +798,7 @@ struct wrap_ndis_device {
 	BOOLEAN stats_enabled;
 	struct ndis_wireless_stats ndis_stats;
 
-	struct work_struct tx_work;
+	work_struct_t tx_work;
 	struct ndis_packet *tx_ring[TX_RING_SIZE];
 	u8 tx_ring_start;
 	u8 tx_ring_end;
@@ -813,8 +813,8 @@ struct wrap_ndis_device {
 	ULONG packet_filter;
 
 	BOOLEAN use_sg_dma;
-	u8 tx_dma_count;
-	dma_addr_t *tx_dma_addr;
+	ULONG dma_map_count;
+	dma_addr_t *dma_map_addr;
 
 	int hangcheck_interval;
 	struct timer_list hangcheck_timer;
@@ -832,7 +832,7 @@ struct wrap_ndis_device {
 	mac_address mac;
 	struct proc_dir_entry *procfs_iface;
 
-	struct work_struct wrap_ndis_work;
+	work_struct_t wrap_ndis_work;
 	unsigned long wrap_ndis_pending_work;
 	UINT attributes;
 	int iw_auth_set;
@@ -864,7 +864,7 @@ struct ndis_pmkid_candidate_list {
 	struct ndis_pmkid_candidate candidates[1];
 };
 
-irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs);
+irqreturn_t ndis_isr(int irq, void *data ISR_PT_REGS_PARAM_DECL);
 void init_nmb_functions(struct ndis_miniport_block *nmb);
 
 int ndis_init(void);
