@@ -60,7 +60,7 @@ static struct nt_list wrap_mdl_list;
 static struct tasklet_struct kdpc_work;
 static void kdpc_worker(unsigned long dummy);
 #else
-static struct work_struct kdpc_work;
+static work_struct_t kdpc_work;
 static void kdpc_worker(void *data);
 #endif
 
@@ -79,7 +79,7 @@ struct bus_driver {
 
 static struct nt_list bus_driver_list;
 
-static struct work_struct ntos_work_item_work;
+static work_struct_t ntos_work_item_work;
 static struct nt_list ntos_work_item_list;
 static NT_SPIN_LOCK ntos_work_item_list_lock;
 static void ntos_work_item_worker(void *data);
@@ -108,7 +108,7 @@ WIN_SYMBOL_MAP("KeTickCount", &jiffies)
 WIN_SYMBOL_MAP("NlsMbCodePageTag", FALSE)
 
 #ifdef USE_OWN_NTOS_WORKQUEUE
-struct workqueue_struct *ntos_wq;
+workqueue_struct_t *ntos_wq;
 #endif
 
 int ntoskernel_init(void)
@@ -130,9 +130,9 @@ int ntoskernel_init(void)
 #ifdef KDPC_TASKLET
 	tasklet_init(&kdpc_work, kdpc_worker, 0);
 #else
-	INIT_WORK(&kdpc_work, kdpc_worker, NULL);
+	initialize_work(&kdpc_work, kdpc_worker, NULL);
 #endif
-	INIT_WORK(&ntos_work_item_work, ntos_work_item_worker, NULL);
+	initialize_work(&ntos_work_item_work, ntos_work_item_worker, NULL);
 	nt_spin_lock_init(&timer_lock);
 	InitializeListHead(&wrap_timer_list);
 
@@ -376,6 +376,7 @@ static int add_bus_driver(const char *name)
 	}
 	memset(bus_driver, 0, sizeof(*bus_driver));
 	strncpy(bus_driver->name, name, sizeof(bus_driver->name));
+	bus_driver->name[sizeof(bus_driver->name)-1] = 0;
 	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	InsertTailList(&bus_driver_list, &bus_driver->list);
 	nt_spin_unlock_irql(&ntoskernel_lock, irql);
@@ -1779,7 +1780,7 @@ static int thread_trampoline(void *data)
 #endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 	strncpy(current->comm, "windisdrvr", sizeof(current->comm));
-	current->comm[sizeof(current->comm) - 1] = 0;
+	current->comm[sizeof(current->comm)-1] = 0;
 #endif
 	DBGTRACE2("thread: %p, task: %p (%d)", thread, thread->task,
 		  thread->pid);
@@ -1981,7 +1982,7 @@ wstdcall void *WIN_FUNC(MmAllocateContiguousMemorySpecifyCache,5)
 {
 	void *addr;
 	size_t page_length = ((size + PAGE_SIZE - 1) / PAGE_SIZE) * PAGE_SIZE;
-	DBGTRACE2("%lu, %lu, %Lu, %Lu, %Lu, %d", size, page_length,
+	DBGTRACE2("%lu, %u, %Lu, %Lu, %Lu, %d", size, page_length,
 		  lowest, highest, boundary, cache_type);
 	addr = ExAllocatePoolWithTag(NonPagedPool, page_length, 0);
 	DBGTRACE2("%p", addr);
@@ -2058,7 +2059,7 @@ struct mdl *allocate_init_mdl(void *virt, ULONG length)
 		MmInitializeMdl(mdl, virt, length);
 		/* mark the MDL as allocated from cache pool so when
 		 * it is freed, we free it back to the pool */
-		mdl->flags = MDL_CACHE_ALLOCATED;
+		mdl->flags = MDL_ALLOCATED_FIXED_SIZE | MDL_CACHE_ALLOCATED;
 	} else {
 		wrap_mdl =
 			kmalloc(sizeof(*wrap_mdl) + mdl_size, gfp_irql());
@@ -2072,6 +2073,7 @@ struct mdl *allocate_init_mdl(void *virt, ULONG length)
 		nt_spin_unlock_irql(&ntoskernel_lock, irql);
 		memset(mdl, 0, mdl_size);
 		MmInitializeMdl(mdl, virt, length);
+		mdl->flags = MDL_ALLOCATED_FIXED_SIZE;
 	}
 	return mdl;
 }
@@ -2318,6 +2320,7 @@ wstdcall NTSTATUS WIN_FUNC(ZwCreateFile,11)
 			memset(bin_file, 0, sizeof(*bin_file));
 			strncpy(bin_file->name, file_basename,
 				sizeof(bin_file->name));
+			bin_file->name[sizeof(bin_file->name)-1] = 0;
 			bin_file->data = vmalloc(*size);
 			if (bin_file->data) {
 				memset(bin_file->data, 0, *size);
@@ -2374,7 +2377,7 @@ wstdcall NTSTATUS WIN_FUNC(ZwReadFile,9)
 	}
 	fo = HANDLE_TO_OBJECT(coh);
 	file = fo->wrap_bin_file;
-	DBGTRACE2("file: %s (%d)", file->name, file->size);
+	DBGTRACE2("file: %s (%u)", file->name, file->size);
 	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	if (byte_offset)
 		offset = *byte_offset;
@@ -2409,14 +2412,14 @@ wstdcall NTSTATUS WIN_FUNC(ZwWriteFile,9)
 	}
 	fo = HANDLE_TO_OBJECT(coh);
 	file = fo->wrap_bin_file;
-	DBGTRACE2("file: %d, %d", file->size, length);
+	DBGTRACE2("file: %u, %u", file->size, length);
 	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	if (byte_offset)
 		offset = *byte_offset;
 	else
 		offset = fo->current_byte_offset;
 	if (length + offset > file->size) {
-		WARNING("%lu, %d", length + offset, file->size);
+		WARNING("%lu, %u", length + offset, (unsigned int)file->size);
 		/* TODO: implement writing past end of current size */
 		iosb->status = STATUS_FAILURE;
 		iosb->info = 0;
@@ -2484,7 +2487,7 @@ wstdcall NTSTATUS WIN_FUNC(ZwQueryInformationFile,5)
 	switch (class) {
 	case FileNameInformation:
 		fni = info;
-		fni->length = min(length, (typeof (length))coh->name.length);
+		fni->length = min(length, (typeof(length))coh->name.length);
 		memcpy(fni->name, coh->name.buf, fni->length);
 		iosb->status = STATUS_SUCCESS;
 		iosb->info = fni->length;
