@@ -63,7 +63,6 @@
 /* pci functions in 2.6 kernels have problems allocating dma buffers,
  * but seem to work fine with dma functions
  */
-typedef struct workqueue_struct *workqueue;
 #include <asm/dma-mapping.h>
 
 #define PCI_DMA_ALLOC_COHERENT(pci_dev,size,dma_handle)			\
@@ -95,39 +94,6 @@ typedef struct workqueue_struct *workqueue;
 #define UNMAP_SG(dev, sglist, nents, direction)		\
 	pci_unmap_sg(dev, sglist, nents, direction)
 
-/* 2.4 kernels don't have workqueues, but we need them */
-struct workqueue_struct {
-	spinlock_t lock;
-	wait_queue_head_t waitq_head;
-	/* how many work_structs pending? */
-	int pending;
-	const char *name;
-	int pid;
-	/* list of work_structs pending */
-	struct list_head work_list;
-};
-
-struct work_struct {
-	struct list_head list;
-	void (*func)(void *data);
-	void *data;
-	/* whether/on which workqueue scheduled */
-	struct workqueue_struct *workq;
-};
-
-#define INIT_WORK(work_struct, worker_func, worker_data)	\
-	do {							\
-		(work_struct)->func = worker_func;		\
-		(work_struct)->data = worker_data;		\
-		(work_struct)->workq = NULL;			\
-	} while (0)
-
-struct workqueue_struct *create_singlethread_workqueue(const char *name);
-void destroy_workqueue(struct workqueue_struct *workq);
-void queue_work(struct workqueue_struct *workq,
-		struct work_struct *work_struct) wfastcall;
-void cancel_delayed_work(struct work_struct *work_struct);
-
 #include <linux/smp_lock.h>
 
 /* RedHat kernels #define irqs_disabled this way */
@@ -216,6 +182,54 @@ do {									\
 })
 #endif
 
+#ifdef USE_OWN_WQ
+
+typedef struct {
+	spinlock_t lock;
+	wait_queue_head_t waitq_head;
+	/* how many work_structs pending? */
+	int pending;
+	const char *name;
+	int pid;
+	/* list of work_structs pending */
+	struct list_head work_list;
+} workqueue_struct_t;
+
+typedef struct {
+	struct list_head list;
+	void (*func)(void *data);
+	void *data;
+	/* whether/on which workqueue scheduled */
+	workqueue_struct_t *workq;
+} work_struct_t;
+
+#define initialize_work(work_struct, worker_func, worker_data)	\
+	do {							\
+		(work_struct)->func = worker_func;		\
+		(work_struct)->data = worker_data;		\
+		(work_struct)->workq = NULL;			\
+	} while (0)
+
+#undef create_singlethread_workqueue
+#define create_singlethread_workqueue wrap_create_wq
+#define destroy_workqueue wrap_destroy_wq
+#define queue_work wrap_queue_work
+#define cancel_delayed_work wrap_cancel_delayed_work
+
+workqueue_struct_t *wrap_create_wq(const char *name);
+void wrap_destroy_wq(workqueue_struct_t *workq);
+void wrap_queue_work(workqueue_struct_t *workq, work_struct_t *work) wfastcall;
+void wrap_cancel_delayed_work(work_struct_t *work);
+
+#else // USE_OWN_WQ
+
+typedef struct workqueue_struct workqueue_struct_t;
+typedef struct work_struct work_struct_t;
+#define initialize_work INIT_WORK
+
+#endif // USE_OWN_WQ
+
+
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
 #define WRAP_MODULE_PARM_INT(name, perm) module_param(name, int, perm)
 #define WRAP_MODULE_PARM_STRING(name, perm) module_param(name, charp, perm)
@@ -332,6 +346,14 @@ typedef u32 pm_message_t;
 #define netdev_priv(dev)  ((dev)->priv)
 #endif
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,18)
+#define ISR_PT_REGS_PARAM_DECL
+#define ISR_PT_REGS_ARG
+#else
+#define ISR_PT_REGS_PARAM_DECL , struct pt_regs *regs
+#define ISR_PT_REGS_ARG , NULL
+#endif
+
 #define memcpy_skb(skb, from, length)			\
 	memcpy(skb_put(skb, length), from, length)
 
@@ -463,7 +485,7 @@ struct wrap_device_setting {
 
 struct wrap_bin_file {
 	char name[MAX_DRIVER_NAME_LEN];
-	int size;
+	size_t size;
 	void *data;
 };
 
@@ -474,9 +496,9 @@ struct wrap_driver {
 	struct driver_object *drv_obj;
 	char name[MAX_DRIVER_NAME_LEN];
 	char version[MAX_SETTING_VALUE_LEN];
-	unsigned int num_pe_images;
+	unsigned short num_pe_images;
 	struct pe_image pe_images[MAX_DRIVER_PE_IMAGES];
-	int num_bin_files;
+	unsigned short num_bin_files;
 	struct wrap_bin_file *bin_files;
 	struct nt_list wrap_devices;
 	struct wrap_ndis_driver *ndis_driver;
@@ -536,15 +558,7 @@ struct wrap_device {
 	(WRAP_DEVICE(dev_bus) == WRAP_BLUETOOTH_DEVICE1 ||	\
 	 WRAP_DEVICE(dev_bus) == WRAP_BLUETOOTH_DEVICE2)
 
-/* Some drivers use worker entries to complete functions called from
- * within other worker threads. So we should have separate workqueues
- * to make sure worker entries run properly */
-
-#define USE_OWN_WORKQUEUE 1
-
-#ifdef USE_OWN_WORKQUEUE
-
-extern struct workqueue_struct *wrap_wq;
+extern workqueue_struct_t *wrap_wq;
 #define schedule_ndis_work(work_struct) queue_work(ndis_wq, (work_struct))
 #define schedule_wrap_work(work_struct) queue_work(wrap_wq, (work_struct))
 
@@ -552,23 +566,18 @@ extern struct workqueue_struct *wrap_wq;
  * it are not supposed to wait; however, it helps to have separate
  * workqueue so keyboard etc. work when kernel crashes */
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,41)
+#ifdef USE_OWN_WQ
 #define USE_OWN_NTOS_WORKQUEUE 1
 #endif
 
 //#define USE_OWN_NTOS_WORKQUEUE 1
 #ifdef USE_OWN_NTOS_WORKQUEUE
-extern struct workqueue_struct *ntos_wq;
+extern workqueue_struct_t *ntos_wq;
 #define schedule_ntos_work(work_struct) queue_work(ntos_wq, (work_struct))
 #else
 #define schedule_ntos_work(work_struct) schedule_work(work_struct)
 #endif
 
-#else // USE_OWN_WORKQUEUE
-#define schedule_ndis_work(work_struct) schedule_work(work_struct)
-#define schedule_wrap_work(work_struct) schedule_work(work_struct)
-#endif // USE_OWN_WORKQUEUE
-
 int ntoskernel_init(void);
 void ntoskernel_exit(void);
 int ntoskernel_init_device(struct wrap_device *wd);
@@ -589,6 +598,9 @@ int rtl_init(void);
 void rtl_exit(void);
 int wrap_procfs_init(void);
 void wrap_procfs_remove(void);
+
+int link_pe_images(struct pe_image *pe_image, unsigned short n);
+
 int stricmp(const char *s1, const char *s2);
 void dump_bytes(const char *name, const u8 *from, int len);
 struct mdl *allocate_init_mdl(void *virt, ULONG length);
@@ -894,8 +906,10 @@ do {									\
 			__asm__ __volatile__(			\
 				LOCK_PREFIX oper "q %q0\n\t"	\
 				: "+m" (var));			\
-		else						\
-			ERROR("invalid size %d", (int)size);	\
+		else {						\
+			extern void _invalid_op_size_(void);	\
+			_invalid_op_size_();			\
+		}						\
 	} while (0)
 
 #define atomic_inc_var_size(var, size) atomic_unary_op(var, size, "inc")
@@ -936,8 +950,18 @@ do {									\
 
 static inline ULONG SPAN_PAGES(void *ptr, SIZE_T length)
 {
+	/* all allocations in ndiswrapper are with kmalloc, so memory
+	 * at ptr is physically contiguous - which can be mapped to
+	 * DMA / physicall address with one register */
+#if 0
 	return PAGE_ALIGN(((unsigned long)ptr & (PAGE_SIZE - 1)) + length)
 			  >> PAGE_SHIFT;
+#else
+	if (length)
+		return 1;
+	else
+		return 0;
+#endif
 }
 
 #ifdef CONFIG_X86_64
