@@ -39,12 +39,13 @@ wstdcall int WIN_FUNC(IoIsWdmVersionAvailable,2)
 	(UCHAR major, UCHAR minor)
 {
 	IOENTER("%d, %x", major, minor);
-	if (major == 1 &&
-	    (minor == 0x30 || // Windows 2003
-	     minor == 0x20 || // Windows XP
-	     minor == 0x10)) // Windows 2000
-		IOEXIT(return 1);
-	IOEXIT(return 0);
+	if ((major == 6 && minor == 0x00) || // Vista
+	    (major == 1 &&
+	     (minor == 0x30 || // Windows 2003
+	      minor == 0x20 || // Windows XP
+	      minor == 0x10))) // Windows 2000
+		IOEXIT(return TRUE);
+	IOEXIT(return FALSE);
 }
 
 wstdcall BOOLEAN WIN_FUNC(IoIs32bitProcess,1)
@@ -518,9 +519,9 @@ wstdcall NTSTATUS IoInvalidDeviceRequest(struct device_object *dev_obj,
 }
 WIN_FUNC_DECL(IoInvalidDeviceRequest,2)
 
-static irqreturn_t io_irq_th(int irq, void *data, struct pt_regs *pt_regs)
+static irqreturn_t io_irq_isr(int irq, void *data ISR_PT_REGS_PARAM_DECL)
 {
-	struct kinterrupt *interrupt = (struct kinterrupt *)data;
+	struct kinterrupt *interrupt = data;
 	NT_SPIN_LOCK *spinlock;
 	BOOLEAN ret;
 
@@ -559,7 +560,7 @@ wstdcall NTSTATUS WIN_FUNC(IoConnectInterrupt,11)
 	InitializeListHead(&interrupt->list);
 	interrupt->synch_irql = synch_irql;
 	interrupt->interrupt_mode = interrupt_mode;
-	if (request_irq(vector, io_irq_th, shareable ? SA_SHIRQ : 0,
+	if (request_irq(vector, io_irq_isr, shareable ? SA_SHIRQ : 0,
 			"io_irq", interrupt)) {
 		WARNING("request for irq %d failed", vector);
 		IOEXIT(return STATUS_INSUFFICIENT_RESOURCES);
