@@ -18,8 +18,7 @@
 #include "wrapndis.h"
 #include "loader.h"
 
-extern NT_SPIN_LOCK loader_lock;
-extern struct wrap_device *wrap_devices;
+extern struct semaphore loader_mutex;
 
 static NTSTATUS start_pdo(struct device_object *pdo)
 {
@@ -34,8 +33,7 @@ static NTSTATUS start_pdo(struct device_object *pdo)
 	wd->surprise_removed = TRUE;
 	if (ntoskernel_init_device(wd))
 		TRACEEXIT1(return STATUS_FAILURE);
-	DBGTRACE1("%d, %d", WRAP_BUS_TYPE(wd->dev_bus_type), WRAP_USB_BUS);
-	if (wrap_is_usb_bus(wd->dev_bus_type)) {
+	if (wrap_is_usb_bus(wd->dev_bus)) {
 #ifdef CONFIG_USB
 		if (usb_init_device(wd)) {
 			ntoskernel_exit_device(wd);
@@ -44,10 +42,9 @@ static NTSTATUS start_pdo(struct device_object *pdo)
 #endif
 		TRACEEXIT1(return STATUS_SUCCESS);
 	}
-	if (!wrap_is_pci_bus(wd->dev_bus_type))
+	if (!wrap_is_pci_bus(wd->dev_bus))
 		TRACEEXIT1(return STATUS_SUCCESS);
 	pdev = wd->pci.pdev;
-	pci_set_drvdata(pdev, wd);
 	ret = pci_enable_device(pdev);
 	if (ret) {
 		ERROR("couldn't enable PCI device: %x", ret);
@@ -158,6 +155,7 @@ static NTSTATUS start_pdo(struct device_object *pdo)
 
 	DBGTRACE2("resource list count %d, irq: %d",
 		  partial_resource_list->count, pdev->irq);
+	pci_set_drvdata(pdev, wd);
 	TRACEEXIT1(return STATUS_SUCCESS);
 err_regions:
 	pci_release_regions(pdev);
@@ -173,13 +171,13 @@ static void remove_pdo(struct device_object *pdo)
 	struct wrap_device *wd = pdo->reserved;
 
 	ntoskernel_exit_device(wd);
-	if (wrap_is_pci_bus(wd->dev_bus_type)) {
+	if (wrap_is_pci_bus(wd->dev_bus)) {
 		struct pci_dev *pdev = wd->pci.pdev;
 		pci_release_regions(pdev);
 		pci_disable_device(pdev);
 		wd->pci.pdev = NULL;
 		pci_set_drvdata(pdev, NULL);
-	} else if (wrap_is_usb_bus(wd->dev_bus_type)) {
+	} else if (wrap_is_usb_bus(wd->dev_bus)) {
 #ifdef CONFIG_USB
 		usb_exit_device(wd);
 #endif
@@ -267,7 +265,7 @@ wstdcall NTSTATUS pdoDispatchPnp(struct device_object *pdo, struct irp *irp)
 		break;
 	case IRP_MN_QUERY_INTERFACE:
 #ifdef CONFIG_USB
-		if (!wrap_is_usb_bus(wd->dev_bus_type)) {
+		if (!wrap_is_usb_bus(wd->dev_bus)) {
 			status = STATUS_NOT_IMPLEMENTED;
 			break;
 		}
@@ -334,7 +332,7 @@ wstdcall NTSTATUS pdoDispatchPower(struct device_object *pdo, struct irp *irp)
 		power_state = irp_sl->params.power.state;
 		if (power_state.device_state == PowerDeviceD0) {
 			DBGTRACE2("resuming device %p", wd);
-			if (wrap_is_pci_bus(wd->dev_bus_type)) {
+			if (wrap_is_pci_bus(wd->dev_bus)) {
 				pdev = wd->pci.pdev;
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
 				pci_restore_state(pdev);
@@ -348,7 +346,7 @@ wstdcall NTSTATUS pdoDispatchPower(struct device_object *pdo, struct irp *irp)
 			}
 		} else {
 			DBGTRACE2("suspending device %p", wd);
-			if (wrap_is_pci_bus(wd->dev_bus_type)) {
+			if (wrap_is_pci_bus(wd->dev_bus)) {
 				pdev = wd->pci.pdev;
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
 				pci_save_state(pdev);
@@ -483,14 +481,16 @@ NTSTATUS pnp_remove_device(struct wrap_device *wd)
 		if (fdo_drv_obj->unload)
 			LIN2WIN1(fdo_drv_obj->unload, fdo_drv_obj);
 		if (wrap_driver) {
-			nt_spin_lock(&loader_lock);
+			if (down_interruptible(&loader_mutex))
+				WARNING("couldn't obtain loader_mutex");
 			unload_wrap_driver(wrap_driver);
-			nt_spin_unlock(&loader_lock);
+			up(&loader_mutex);
 		} else
 			ERROR("couldn't get wrap_driver");
 		ObDereferenceObject(fdo_drv_obj);
 	}
 	IoDeleteDevice(pdo);
+	unload_wrap_device(wd);
 	TRACEEXIT1(return status);
 }
 
@@ -523,16 +523,13 @@ static struct device_object *alloc_pdo(struct driver_object *drv_obj)
 		return NULL;
 	/* dispatch routines are called as Windows functions */
 	for (i = 0; i <= IRP_MJ_MAXIMUM_FUNCTION; i++)
-		drv_obj->major_func[i] =
-			WIN_FUNC_PTR(IoInvalidDeviceRequest,2);
+		drv_obj->major_func[i] = WIN_FUNC_PTR(IoInvalidDeviceRequest,2);
 	drv_obj->major_func[IRP_MJ_INTERNAL_DEVICE_CONTROL] =
 		WIN_FUNC_PTR(pdoDispatchDeviceControl,2);
 	drv_obj->major_func[IRP_MJ_DEVICE_CONTROL] =
 		WIN_FUNC_PTR(pdoDispatchDeviceControl,2);
-	drv_obj->major_func[IRP_MJ_POWER] =
-		WIN_FUNC_PTR(pdoDispatchPower,2);
-	drv_obj->major_func[IRP_MJ_PNP] =
-		WIN_FUNC_PTR(pdoDispatchPnp,2);
+	drv_obj->major_func[IRP_MJ_POWER] = WIN_FUNC_PTR(pdoDispatchPower,2);
+	drv_obj->major_func[IRP_MJ_PNP] = WIN_FUNC_PTR(pdoDispatchPnp,2);
 	return pdo;
 }
 
@@ -544,18 +541,18 @@ static int wrap_pnp_start_device(struct wrap_device *wd)
 
 	TRACEENTER1("wd: %p", wd);
 
-	if (!((wrap_is_pci_bus(wd->dev_bus_type)) ||
-	      (wrap_is_usb_bus(wd->dev_bus_type)))) {
+	if (!((wrap_is_pci_bus(wd->dev_bus)) ||
+	      (wrap_is_usb_bus(wd->dev_bus)))) {
 		ERROR("bus type %d (%d) not supported",
-		      WRAP_BUS_TYPE(wd->dev_bus_type), wd->dev_bus_type);
+		      WRAP_BUS(wd->dev_bus), wd->dev_bus);
 		TRACEEXIT1(return -EINVAL);
 	}
 
-	if (!((WRAP_DEVICE_TYPE(wd->dev_bus_type) == WRAP_NDIS_DEVICE) ||
-	      (WRAP_DEVICE_TYPE(wd->dev_bus_type) == WRAP_USB_DEVICE) ||
-	      (wrap_is_bluetooth_device(wd->dev_bus_type)))) {
-		ERROR("device type %d (%d) not supported",
-		      WRAP_DEVICE_TYPE(wd->dev_bus_type), wd->dev_bus_type);
+	if (!((WRAP_DEVICE(wd->dev_bus) == WRAP_NDIS_DEVICE) ||
+	      (WRAP_DEVICE(wd->dev_bus) == WRAP_USB_DEVICE) ||
+	      (wrap_is_bluetooth_device(wd->dev_bus)))) {
+		ERROR("device %d (%d) not supported",
+		      WRAP_DEVICE(wd->dev_bus), wd->dev_bus);
 		TRACEEXIT1(return -EINVAL);
 	}
 	driver = load_wrap_driver(wd);
@@ -563,13 +560,12 @@ static int wrap_pnp_start_device(struct wrap_device *wd)
 		return -ENODEV;
 
 	wd->driver = driver;
-	DBGTRACE1("dev type: %d, bus type: %d, %d",
-		  WRAP_DEVICE_TYPE(wd->dev_bus_type),
-		  WRAP_BUS_TYPE(wd->dev_bus_type), wd->dev_bus_type);
+	DBGTRACE1("dev type: %d, bus type: %d, %d", WRAP_DEVICE(wd->dev_bus),
+		  WRAP_BUS(wd->dev_bus), wd->dev_bus);
 	/* first create pdo */
-	if (wrap_is_pci_bus(wd->dev_bus_type))
+	if (wrap_is_pci_bus(wd->dev_bus))
 		pdo_drv_obj = find_bus_driver("PCI");
-	else // if (wrap_is_usb_bus(wd->dev_bus_type))
+	else // if (wrap_is_usb_bus(wd->dev_bus))
 		pdo_drv_obj = find_bus_driver("USB");
 	if (!pdo_drv_obj)
 		return -EINVAL;
@@ -578,7 +574,7 @@ static int wrap_pnp_start_device(struct wrap_device *wd)
 		return -ENOMEM;
 	wd->pdo = pdo;
 	pdo->reserved = wd;
-	if (WRAP_DEVICE_TYPE(wd->dev_bus_type) == WRAP_NDIS_DEVICE) {
+	if (WRAP_DEVICE(wd->dev_bus) == WRAP_NDIS_DEVICE) {
 		if (init_ndis_driver(driver->drv_obj)) {
 			IoDeleteDevice(pdo);
 			return -EINVAL;
@@ -606,12 +602,21 @@ static int wrap_pnp_start_device(struct wrap_device *wd)
 int wrap_pnp_start_pci_device(struct pci_dev *pdev,
 			      const struct pci_device_id *ent)
 {
+	struct load_device load_device;
 	struct wrap_device *wd;
 
 	TRACEENTER1("called for %04x:%04x:%04x:%04x", pdev->vendor,
 		    pdev->device, pdev->subsystem_vendor,
 		    pdev->subsystem_device);
-	wd = &wrap_devices[ent->driver_data];
+
+	load_device.bus = WRAP_PCI_BUS;
+	load_device.vendor = pdev->vendor;
+	load_device.device = pdev->device;
+	load_device.subvendor = pdev->subsystem_vendor;
+	load_device.subdevice = pdev->subsystem_device;
+	wd = load_wrap_device(&load_device);
+	if (!wd)
+		TRACEEXIT1(return -ENODEV);
 	wd->pci.pdev = pdev;
 	return wrap_pnp_start_device(wd);
 }
@@ -655,45 +660,51 @@ void *wrap_pnp_start_usb_device(struct usb_device *udev,
 {
 	struct wrap_device *wd;
 	int ret;
-	wd = &wrap_devices[usb_id->driver_info];
-	TRACEENTER1("%04x, %04x, %x, %x, %x, %x, %x, %x, %x, %p",
-		    usb_id->idVendor, usb_id->idProduct,
-		    usb_id->bDeviceClass, usb_id->bDeviceSubClass,
-		    usb_id->bDeviceProtocol, usb_id->bInterfaceClass,
-		    usb_id->bDeviceProtocol, usb_id->bInterfaceClass,
-		    usb_id->bInterfaceSubClass, wd->usb.intf);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+	struct usb_device *udev = interface_to_usbdev(intf);
+#else
+	struct usb_interface *intf = usb_ifnum_to_if(udev, ifnum);
+#endif
+	TRACEENTER1("%04x, %04x, %04x", udev->descriptor.idVendor,
+		    udev->descriptor.idProduct, udev->descriptor.bDeviceClass);
+
 	/* USB device (e.g., RNDIS) may have multiple interfaces;
 	  initialize one interface only (is there a way to know which
 	  of these interfaces is for network?) */
-	if (wd->usb.intf) {
-		DBGTRACE1("device already initialized: %p", wd->usb.intf);
+
+	if ((wd = get_wrap_device(udev, WRAP_USB_BUS))) {
+		DBGTRACE1("device already initialized: %p", wd);
 		usb_set_intfdata(intf, NULL);
 		ret = 0;
 	} else {
-		/* some devices (e.g., TI 4150, RNDIS) need full reset */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-		ret = usb_reset_device(interface_to_usbdev(intf));
-		if (ret)
-			WARNING("reset failed: %d", ret);
-		wd->usb.udev = interface_to_usbdev(intf);
-		usb_set_intfdata(intf, wd);
-		wd->usb.intf = intf;
-#else
-		ret = usb_reset_device(udev);
-		if (ret)
-			WARNING("reset failed: %d", ret);
-		wd->usb.udev = udev;
-		wd->usb.intf = usb_ifnum_to_if(udev, ifnum);
-#endif
-		ret = wrap_pnp_start_device(wd);
+		struct load_device load_device;
+
+		load_device.bus = WRAP_USB_BUS;
+		load_device.vendor = udev->descriptor.idVendor;
+		load_device.device = udev->descriptor.idProduct;
+		load_device.subvendor = 0;
+		load_device.subdevice = 0;
+		wd = load_wrap_device(&load_device);
+		DBGTRACE2("%p", wd);
+		if (wd) {
+			/* some devices (e.g., TI 4150, RNDIS) need
+			 * full reset */
+			ret = usb_reset_device(udev);
+			if (ret)
+				WARNING("reset failed: %d", ret);
+			usb_set_intfdata(intf, wd);
+			wd->usb.intf = intf;
+			wd->usb.udev = udev;
+			ret = wrap_pnp_start_device(wd);
+		} else
+			ret = -ENODEV;
 	}
 
 	DBGTRACE2("ret: %d", ret);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	if (ret) {
-		wd->usb.intf = NULL;
-		TRACEEXIT1(return -EINVAL);
-	} else
+	if (ret)
+		TRACEEXIT1(return ret);
+	else
 		return 0;
 #else
 	if (ret)
@@ -704,7 +715,7 @@ void *wrap_pnp_start_usb_device(struct usb_device *udev,
 }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-void wrap_pnp_remove_usb_device(struct usb_interface *intf)
+void __devexit wrap_pnp_remove_usb_device(struct usb_interface *intf)
 {
 	struct wrap_device *wd;
 
@@ -716,7 +727,6 @@ void wrap_pnp_remove_usb_device(struct usb_interface *intf)
 	if (wd->surprise_removed == TRUE)
 		wd->usb.intf = NULL;
 	pnp_remove_device(wd);
-	wd->usb.intf = NULL;
 }
 
 int wrap_pnp_suspend_usb_device(struct usb_interface *intf, pm_message_t state)
@@ -748,7 +758,7 @@ int wrap_pnp_resume_usb_device(struct usb_interface *intf)
 
 #else
 
-void wrap_pnp_remove_usb_device(struct usb_device *udev, void *ptr)
+void __devexit wrap_pnp_remove_usb_device(struct usb_device *udev, void *ptr)
 {
 	struct wrap_device *wd = ptr;
 	struct usb_interface *intf;
@@ -760,7 +770,6 @@ void wrap_pnp_remove_usb_device(struct usb_device *udev, void *ptr)
 	if (wd->surprise_removed == TRUE)
 		wd->usb.intf = NULL;
 	pnp_remove_device(wd);
-	wd->usb.intf = NULL;
 }
 #endif
 
