@@ -297,9 +297,11 @@ static int procfs_write_ndis_settings(struct file *file, const char *buf,
 			return -EINVAL;
 		p++;
 		i = simple_strtol(p, NULL, 10);
-		hangcheck_interval = i;
 		hangcheck_del(wnd);
-		hangcheck_add(wnd);
+		if (i > 0) {
+			wnd->hangcheck_interval = i * HZ;
+			hangcheck_add(wnd);
+		}
 	} else if (!strcmp(setting, "suspend")) {
 		if (!p)
 			return -EINVAL;
@@ -307,7 +309,7 @@ static int procfs_write_ndis_settings(struct file *file, const char *buf,
 		i = simple_strtol(p, NULL, 10);
 		if (i <= 0 || i > 3)
 			return -EINVAL;
-		if (wrap_is_pci_bus(wnd->wd->dev_bus_type))
+		if (wrap_is_pci_bus(wnd->wd->dev_bus))
 			i = wrap_pnp_suspend_pci_device(wnd->wd->pci.pdev,
 							PMSG_SUSPEND);
 		else
@@ -320,7 +322,7 @@ static int procfs_write_ndis_settings(struct file *file, const char *buf,
 		if (i)
 			return -EINVAL;
 	} else if (!strcmp(setting, "resume")) {
-		if (wrap_is_pci_bus(wnd->wd->dev_bus_type))
+		if (wrap_is_pci_bus(wnd->wd->dev_bus))
 			i = wrap_pnp_resume_pci_device(wnd->wd->pci.pdev);
 		else
 #if defined(CONFIG_USB) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
@@ -353,26 +355,25 @@ static int procfs_write_ndis_settings(struct file *file, const char *buf,
 
 int wrap_procfs_add_ndis_device(struct wrap_ndis_device *wnd)
 {
-	struct net_device *dev = wnd->net_dev;
-	struct proc_dir_entry *proc_iface, *procfs_entry;
+	struct proc_dir_entry *procfs_entry;
 
-	wnd->procfs_iface = NULL;
 	if (wrap_procfs_entry == NULL)
 		return -ENOMEM;
 
-	proc_iface = proc_mkdir(dev->name, wrap_procfs_entry);
-
-	wnd->procfs_iface = proc_iface;
-
-	if (proc_iface == NULL) {
+	if (wnd->procfs_iface) {
+		ERROR("%s already registered?", wnd->netdev_name);
+		return -EINVAL;
+	}
+	wnd->procfs_iface = proc_mkdir(wnd->netdev_name, wrap_procfs_entry);
+	if (wnd->procfs_iface == NULL) {
 		ERROR("couldn't create proc directory");
 		return -ENOMEM;
 	}
-	proc_iface->uid = proc_uid;
-	proc_iface->gid = proc_gid;
+	wnd->procfs_iface->uid = proc_uid;
+	wnd->procfs_iface->gid = proc_gid;
 
 	procfs_entry = create_proc_entry("hw", S_IFREG | S_IRUSR | S_IRGRP,
-					 proc_iface);
+					 wnd->procfs_iface);
 	if (procfs_entry == NULL) {
 		ERROR("couldn't create proc entry for 'hw'");
 		return -ENOMEM;
@@ -384,7 +385,7 @@ int wrap_procfs_add_ndis_device(struct wrap_ndis_device *wnd)
 	}
 
 	procfs_entry = create_proc_entry("stats", S_IFREG | S_IRUSR | S_IRGRP,
-					 proc_iface);
+					 wnd->procfs_iface);
 	if (procfs_entry == NULL) {
 		ERROR("couldn't create proc entry for 'stats'");
 		return -ENOMEM;
@@ -396,7 +397,7 @@ int wrap_procfs_add_ndis_device(struct wrap_ndis_device *wnd)
 	}
 
 	procfs_entry = create_proc_entry("encr", S_IFREG | S_IRUSR | S_IRGRP,
-					 proc_iface);
+					 wnd->procfs_iface);
 	if (procfs_entry == NULL) {
 		ERROR("couldn't create proc entry for 'encr'");
 		return -ENOMEM;
@@ -409,7 +410,7 @@ int wrap_procfs_add_ndis_device(struct wrap_ndis_device *wnd)
 
 	procfs_entry = create_proc_entry("settings", S_IFREG |
 					 S_IRUSR | S_IRGRP |
-					 S_IWUSR | S_IWGRP, proc_iface);
+					 S_IWUSR | S_IWGRP, wnd->procfs_iface);
 	if (procfs_entry == NULL) {
 		ERROR("couldn't create proc entry for 'settings'");
 		return -ENOMEM;
@@ -425,7 +426,6 @@ int wrap_procfs_add_ndis_device(struct wrap_ndis_device *wnd)
 
 void wrap_procfs_remove_ndis_device(struct wrap_ndis_device *wnd)
 {
-	struct net_device *dev = wnd->net_dev;
 	struct proc_dir_entry *procfs_iface = wnd->procfs_iface;
 
 	if (procfs_iface == NULL)
@@ -435,7 +435,7 @@ void wrap_procfs_remove_ndis_device(struct wrap_ndis_device *wnd)
 	remove_proc_entry("encr", procfs_iface);
 	remove_proc_entry("settings", procfs_iface);
 	if (wrap_procfs_entry != NULL)
-		remove_proc_entry(dev->name, wrap_procfs_entry);
+		remove_proc_entry(wnd->netdev_name, wrap_procfs_entry);
 	wnd->procfs_iface = NULL;
 }
 
