@@ -129,14 +129,14 @@ static void usb_kill_urb(struct urb *urb)
 static struct nt_list wrap_urb_complete_list;
 static NT_SPIN_LOCK wrap_urb_complete_list_lock;
 
-/* use tasklet instead worker to process completed urbs */
+/* use tasklet instead of worker to process completed urbs */
 #define USB_TASKLET 1
 
 #ifdef USB_TASKLET
 static struct tasklet_struct wrap_urb_complete_work;
 static void wrap_urb_complete_worker(unsigned long dummy);
 #else
-static struct work_struct wrap_urb_complete_work;
+static work_struct_t wrap_urb_complete_work;
 static void wrap_urb_complete_worker(void *dummy);
 #endif
 
@@ -149,7 +149,7 @@ int usb_init(void)
 #ifdef USB_TASKLET
 	tasklet_init(&wrap_urb_complete_work, wrap_urb_complete_worker, 0);
 #else
-	INIT_WORK(&wrap_urb_complete_work, wrap_urb_complete_worker, NULL);
+	initialize_work(&wrap_urb_complete_work, wrap_urb_complete_worker, NULL);
 #endif
 #ifdef USB_DEBUG
 	urb_id = 0;
@@ -385,7 +385,7 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 	urb->transfer_buffer_length = buf_len;
 	if (buf_len && buf && (!virt_addr_valid(buf)
 #if defined(CONFIG_HIGHMEM) || defined(CONFIG_HIGHMEM4G)
-		    || PageHighMem(virt_to_page(buf))
+			       || PageHighMem(virt_to_page(buf))
 #endif
 		    )) {
 		urb->transfer_buffer =
@@ -475,7 +475,7 @@ static void int_urb_unlink_complete(struct urb *urb)
 
 static void wrap_urb_complete(struct urb *urb)
 #else
-static void wrap_urb_complete(struct urb *urb, struct pt_regs *regs)
+static void wrap_urb_complete(struct urb *urb ISR_PT_REGS_PARAM_DECL)
 #endif
 {
 	struct irp *irp;
