@@ -15,88 +15,110 @@
 
 #include "ntoskernel.h"
 
-spinlock_t workq_lock = SPIN_LOCK_UNLOCKED;
-
 /* workqueue implementation for 2.4 kernels */
 
-static int workqueue_thread(void *data)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,8)
+#define SIG_LOCK(t) (&(t)->sigmask_lock)
+#else
+#define SIG_LOCK(t) (&(t)->sighand->siglock)
+#endif
+
+static int workq_thread(void *data)
 {
-	struct workqueue_struct *workq = data;
-	struct work_struct *work;
+	workqueue_struct_t *workq = data;
+	work_struct_t *work;
+	unsigned long flags;
 
 	lock_kernel();
 	strncpy(current->comm, workq->name, sizeof(current->comm));
 	current->comm[sizeof(current->comm) - 1] = 0;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,7)
 	daemonize();
+#else
+	daemonize(workq->name);
+#endif
 	unlock_kernel();
+#ifdef PF_NOFREEZE
+	current->flags |= PF_NOFREEZE;
+	set_user_nice(current, -5);
+#endif
 	while (1) {
 		if (wait_event_interruptible(workq->waitq_head,
-					     workq->pending != 0)) {
-			/* we don't want to terminate thread */
+					     workq->pending)) {
+			/* TODO: deal with signal */
+			spin_lock_irq(SIG_LOCK(current));
 			flush_signals(current);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,7)
+			recalc_sigpending(current);
+#else
+			recalc_sigpending();
+#endif
+			spin_unlock_irq(SIG_LOCK(current));
 			continue;
 		}
-		spin_lock_bh(&workq->lock);
+		spin_lock_irqsave(&workq->lock, flags);
 		if (workq->pending-- < 0)
 			break;
 		if (list_empty(&workq->work_list))
 			work = NULL;
 		else {
-			struct list_head *entry;
-			entry = workq->work_list.next;
-			work = list_entry(entry, struct work_struct, list);
-			list_del_init(entry);
-			work->workq = NULL;
+			struct list_head *entry = workq->work_list.next;
+			work = list_entry(entry, work_struct_t, list);
+			list_del(entry);
+			if (work->workq) {
+				assert(work->workq == workq);
+				work->workq = NULL;
+			} else
+				work = NULL;
 		}
-		spin_unlock_bh(&workq->lock);
+		spin_unlock_irqrestore(&workq->lock, flags);
 		if (work)
 			work->func(work->data);
 	}
 	/* set workq for each work to NULL so if cancel_delayed_work
 	 * is called later, it won't access workq */
-	spin_lock_bh(&workq_lock);
 	list_for_each_entry(work, &workq->work_list, list) {
 		work->workq = NULL;
 	}
-	spin_unlock_bh(&workq_lock);
-	spin_unlock_bh(&workq->lock);
+	spin_unlock_irqrestore(&workq->lock, flags);
 	WORKTRACE("%s exiting", workq->name);
 	workq->pid = 0;
 	return 0;
 }
 
-wfastcall void queue_work(struct workqueue_struct *workq,
-			  struct work_struct *work)
+wfastcall void wrap_queue_work(workqueue_struct_t *workq, work_struct_t *work)
 {
-	spin_lock_bh(&workq->lock);
+	unsigned long flags;
+
+	spin_lock_irqsave(&workq->lock, flags);
 	if (!work->workq) {
 		work->workq = workq;
 		list_add_tail(&work->list, &workq->work_list);
 		workq->pending++;
 		wake_up_interruptible(&workq->waitq_head);
 	}
-	spin_unlock_bh(&workq->lock);
+	spin_unlock_irqrestore(&workq->lock, flags);
 }
 
-void cancel_delayed_work(struct work_struct *work)
+void wrap_cancel_delayed_work(work_struct_t *work)
 {
-	struct workqueue_struct *workq;
-	spin_lock_bh(&workq_lock);
-	workq = work->workq;
-	if (workq) {
-		spin_lock_bh(&workq->lock);
+	workqueue_struct_t *workq;
+	unsigned long flags;
+
+	if ((workq = xchg(&work->workq, NULL))) {
+		spin_lock_irqsave(&workq->lock, flags);
 		list_del(&work->list);
-		/* don't decrement workq->pending here; otherwise, it may
-		 * prematurely terminate the thread, as wait_event
-		 * above checks it without lock */
-		spin_unlock_bh(&workq->lock);
+		/* don't decrement workq->pending here; otherwise, it
+		 * may prematurely terminate the thread, as this work
+		 * may already have been done (pending may have been
+		 * decremented for it) */
+		spin_unlock_irqrestore(&workq->lock, flags);
 	}
-	spin_unlock_bh(&workq_lock);
 }
 
-struct workqueue_struct *create_singlethread_workqueue(const char *name)
+workqueue_struct_t *wrap_create_wq(const char *name)
 {
-	struct workqueue_struct *workq = kmalloc(sizeof(*workq), GFP_KERNEL);
+	workqueue_struct_t *workq = kmalloc(sizeof(*workq), GFP_KERNEL);
 	if (!workq) {
 		WARNING("couldn't allocate memory");
 		return NULL;
@@ -108,7 +130,7 @@ struct workqueue_struct *create_singlethread_workqueue(const char *name)
 	INIT_LIST_HEAD(&workq->work_list);
 	/* we don't need to wait for thread to start, so completion
 	 * not used */
-	workq->pid = kernel_thread(workqueue_thread, workq, 0);
+	workq->pid = kernel_thread(workq_thread, workq, 0);
 	if (workq->pid <= 0) {
 		kfree(workq);
 		WARNING("couldn't start thread %s", name);
@@ -117,7 +139,7 @@ struct workqueue_struct *create_singlethread_workqueue(const char *name)
 	return workq;
 }
 
-void destroy_workqueue(struct workqueue_struct *workq)
+void wrap_destroy_wq(workqueue_struct_t *workq)
 {
 	while (workq->pid) {
 		workq->pending = -1;
