@@ -54,10 +54,10 @@ NDIS_STATUS miniport_reset(struct wrap_ndis_device *wnd)
 
 	TRACEENTER2("wnd: %p", wnd);
 
-	if (down_interruptible(&wnd->ndis_comm_mutex))
+	if (down_interruptible(&wnd->tx_ring_mutex))
 		TRACEEXIT3(return NDIS_STATUS_FAILURE);
-	if (down_interruptible(&wnd->tx_ring_mutex)) {
-		up(&wnd->ndis_comm_mutex);
+	if (down_interruptible(&wnd->ndis_comm_mutex)) {
+		up(&wnd->tx_ring_mutex);
 		TRACEEXIT3(return NDIS_STATUS_FAILURE);
 	}		
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
@@ -763,7 +763,7 @@ static void ndis_poll_controller(struct net_device *dev)
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
 
 	disable_irq(dev->irq);
-	ndis_isr(dev->irq, wnd, NULL);
+	ndis_isr(dev->irq, wnd ISR_PT_REGS_ARG);
 	enable_irq(dev->irq);
 }
 #endif
@@ -850,10 +850,10 @@ static int ndis_set_wol(struct net_device *dev, struct ethtool_wolinfo *wol)
 }
 
 static struct ethtool_ops ndis_ethtool_ops = {
-	.get_drvinfo		= ndis_get_drvinfo,
-	.get_link		= ndis_get_link,
-	.get_wol		= ndis_get_wol,
-	.set_wol		= ndis_set_wol,
+	.get_drvinfo	= ndis_get_drvinfo,
+	.get_link	= ndis_get_link,
+	.get_wol	= ndis_get_wol,
+	.set_wol	= ndis_set_wol,
 };
 #endif
 
@@ -887,7 +887,7 @@ static void update_wireless_stats(struct wrap_ndis_device *wnd)
 	struct ndis_wireless_stats ndis_stats;
 	NDIS_STATUS res;
 	ndis_rssi rssi;
-	unsigned long frag;
+	int qual;
 
 	TRACEENTER2("%p", wnd);
 	if (wnd->stats_enabled == FALSE || !netif_carrier_ok(wnd->net_dev)) {
@@ -898,7 +898,15 @@ static void update_wireless_stats(struct wrap_ndis_device *wnd)
 	if (res == NDIS_STATUS_SUCCESS)
 		iw_stats->qual.level = rssi;
 
-	memset(&ndis_stats, 0, sizeof(ndis_stats));
+	qual = 100 * (rssi - WL_NOISE) / (WL_SIGMAX - WL_NOISE);
+	if (qual < 0)
+		qual = 0;
+	else if (qual > 100)
+		qual = 100;
+
+	iw_stats->qual.noise = WL_NOISE;
+	iw_stats->qual.qual  = qual;
+
 	res = miniport_query_info(wnd, OID_802_11_STATISTICS,
 				  &ndis_stats, sizeof(ndis_stats));
 	if (res != NDIS_STATUS_SUCCESS)
@@ -910,15 +918,6 @@ static void update_wireless_stats(struct wrap_ndis_device *wnd)
 		(unsigned long)ndis_stats.ack_fail +
 		(unsigned long)ndis_stats.frame_dup;
 
-	frag = 6 * (unsigned long)ndis_stats.tx_frag;
-	if (frag)
-		iw_stats->qual.qual =
-			100 - 100 * (((unsigned long)ndis_stats.retry +
-				      2*(unsigned long)ndis_stats.multi_retry +
-				      3*(unsigned long)ndis_stats.failed) /
-				     frag);
-	else
-		iw_stats->qual.qual = 100;
 	TRACEEXIT2(return);
 }
 
@@ -1755,8 +1754,8 @@ static int ndis_remove_device(struct wrap_ndis_device *wnd)
 	unregister_netdev(wnd->net_dev);
 	netif_carrier_off(wnd->net_dev);
 	cancel_delayed_work(&wnd->wrap_ndis_work);
-	/* if device is suspended, but resume failed, tx_ring_mutex is
-	 * already locked */
+	/* if device is suspended, but resume failed, tx_ring_mutex
+	 * may already be locked */
 	down_trylock(&wnd->tx_ring_mutex);
 	tx_pending = wnd->tx_ring_end - wnd->tx_ring_start;
 	if (tx_pending < 0)
@@ -1856,7 +1855,7 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	init_MUTEX(&wnd->ndis_comm_mutex);
 	init_waitqueue_head(&wnd->ndis_comm_wq);
 	wnd->ndis_comm_done = 0;
-	INIT_WORK(&wnd->tx_work, tx_worker, wnd);
+	initialize_work(&wnd->tx_work, tx_worker, wnd);
 	wnd->tx_ring_start = 0;
 	wnd->tx_ring_end = 0;
 	wnd->is_tx_ring_full = 0;
@@ -1865,8 +1864,8 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	wnd->capa.encr = 0;
 	wnd->capa.auth = 0;
 	wnd->attributes = 0;
-	wnd->tx_dma_count = 0;
-	wnd->tx_dma_addr = NULL;
+	wnd->dma_map_count = 0;
+	wnd->dma_map_addr = NULL;
 	wnd->nick[0] = 0;
 	init_timer(&wnd->hangcheck_timer);
 	wnd->scan_timestamp = 0;
@@ -1876,7 +1875,7 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	memset(&wnd->essid, 0, sizeof(wnd->essid));
 	memset(&wnd->encr_info, 0, sizeof(wnd->encr_info));
 	wnd->infrastructure_mode = Ndis802_11Infrastructure;
-	INIT_WORK(&wnd->wrap_ndis_work, wrap_ndis_worker, wnd);
+	initialize_work(&wnd->wrap_ndis_work, wrap_ndis_worker, wnd);
 	wnd->hw_status = 0;
 	if (wd->driver->ndis_driver)
 		wd->driver->ndis_driver->miniport.shutdown = NULL;
